/*

   This Amiga code was inspired by the Linux code, written by
   Svante Gerhard.

   It`s a bit of a no-frills port, so don`t expect menus, different fonts,
   mip-mapping etc. Not yet, anyway.

   Feel free to modify/improve/delete as you choose.

   Mark Howson, Feb 1999 <markh@angband.org>

*/

#define CRAWLVERSION "Crawl 3.20"

#define WIN_COLS 80
#define WIN_ROWS 30

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <ctype.h>

#include <exec/io.h>
#include <exec/ports.h>

#include <intuition/intuition.h>
#include <intuition/intuitionbase.h>
#include <libraries/asl.h>
#include <devices/inputevent.h>
#include <devices/conunit.h>
#include <graphics/gfxbase.h>
#include <graphics/text.h>

#ifdef PPC
#  include <inline/asl.h>
#  include <inline/intuition.h>
#  include <inline/graphics.h>
#  include <inline/exec.h>
#  include <inline/console.h>
#  include <inline/diskfont.h>
#else
#  include <proto/diskfont.h>
#  include <proto/asl.h>
#  include <proto/intuition.h>
#  include <proto/graphics.h>
#  include <proto/exec.h>
#  include <proto/console.h>
#endif

#include "amigalib.h"

struct Library *DiskfontBase = NULL;
struct GfxBase *GfxBase = NULL;
struct IntuitionBase *IntuitionBase = NULL;
struct Library *AslBase = NULL;
struct Device *ConsoleDev = NULL;
struct Library *ConsoleDevice = NULL;
struct Library *SysBase = (Library *)0x4L;

static int start_state = 0;

static struct Screen *ami_scr = NULL;
static struct Window *ami_win = NULL;
static struct RastPort *rp = NULL;
static struct ViewPort *vp = NULL;
static struct TextFont *ami_fnt = NULL;

static struct InputEvent ie;
static struct IOStdReq io_req;

static int pending = 0;
static int keycode = 0;

// Globals holding current text/backg. colors
short FG_COL = WHITE;
short BG_COL = BLACK;

long _stack = 100000;

/* Width, height, baseline of our font */
static int font_w = 8, font_h = 8, font_b = 6;

/* Do we own the font? */
static int font_owner = FALSE;

static int my_x = 0,my_y = 0;

void testcol(int col, char *str)
{
	textcolor(col);
	printf("Test using color %s \n", str);
}

static int setrgb(int col,int red, int green, int blue)
{
	SetRGB4(vp,col,red >> 4,green >> 4, blue >> 4);
}

void lincurses_startup()
{
	struct TextAttr amifont;
	char str[256];
	ULONG reqmode = 0;
	int i;

	if (start_state)
	{
		//fprintf(stderr,"curses_startup() called twice!\n");
		clrscr();
		gotoxy(0,0);
		return;
	}

	atexit(lincurses_shutdown);
	start_state++;

	GfxBase = (struct GfxBase *)OpenLibrary((UBYTE *)"graphics.library",36);
	if (!GfxBase)
	{
		puts("Need graphics.library V36 or better!");
		exit(5);
	}
	DiskfontBase = OpenLibrary( (UBYTE *)"diskfont.library", 0L);
	if (!DiskfontBase)
	{
		puts("Need diskfont.library!");
		exit(5);
	}
	IntuitionBase = (struct IntuitionBase *)OpenLibrary((UBYTE *)"intuition.library",36);
	AslBase = (struct Library *)OpenLibrary((UBYTE *)"asl.library", 0L);

	if (!AslBase)
	{
		puts("Need asl.library!");
		exit(5);
	}

	ie.ie_NextEvent = NULL;
	ie.ie_Class 	 = IECLASS_RAWKEY;
	ie.ie_SubClass  = 0;

	ConsoleDev = (struct Device *)OpenDevice((UBYTE *)"console.device",0xFFFFFFFF,(struct IORequest *)&io_req,0L);
	ConsoleDevice = (struct Library *)io_req.io_Device;

	font_owner = FALSE;

	amifont.ta_Name = "topaz.font";
	amifont.ta_YSize = 0;
	amifont.ta_Flags = FPF_ROMFONT;
	amifont.ta_Style = FS_NORMAL;

	/* Try and read prefs */
	FILE *f = fopen("PROGDIR:Crawl.cfg", "r");
	if (f)
	{
		int size;

		fscanf(f,"%lx\n",&reqmode);
		fscanf(f,"%s\n",&str);
		fscanf(f,"%d\n",&size);

		amifont.ta_YSize = size;
		amifont.ta_Name = str;
		fclose(f);
	}

	/* Screen mode requester */
	struct ScreenModeRequester *req;

	if ( !reqmode && (req = (struct ScreenModeRequester *)AllocAslRequestTags( ASL_ScreenModeRequest, TAG_DONE )))
	{
		if ( AslRequestTags( req, TAG_DONE ))
			reqmode = req->sm_DisplayID;
		FreeAslRequest( req );
	}

	if (!reqmode)
	{
		puts("Must choose a screenmode!");
		exit(5);
	}

	struct FontRequester *freq = NULL;

	if ( !amifont.ta_YSize && (freq = AllocAslRequestTags( ASL_FontRequest, TAG_DONE )))
	{
		if ( AslRequestTags( freq,
				ASLFO_FixedWidthOnly, TRUE,
				TAG_DONE, TAG_DONE ))
		{
			amifont.ta_Name = strdup(freq->fo_Attr.ta_Name);
			amifont.ta_YSize = freq->fo_Attr.ta_YSize;
		}
		FreeAslRequest( freq );
	}

	/* Open Amiga font */
	amifont.ta_Flags = FPF_DISKFONT;
	if (!stricmp(amifont.ta_Name, "topaz.font") && (amifont.ta_YSize == 8 || amifont.ta_YSize == 9) )
		amifont.ta_Flags = FPF_ROMFONT;

	ami_fnt = OpenDiskFont( &amifont );
	ULONG tfont = (ULONG)ami_fnt;

	font_owner = TRUE;
	if (!ami_fnt)
	{
		font_owner = FALSE;

		ami_fnt = GfxBase->DefaultFont;
	}

	font_w = ami_fnt->tf_XSize;
	font_h = ami_fnt->tf_YSize;
	font_b = ami_fnt->tf_Baseline;

	ami_scr = OpenScreenTags( NULL,
										SA_DisplayID, reqmode,
										SA_Depth, 4,
										SA_Type, CUSTOMSCREEN,
										SA_Title, CRAWLVERSION,
										SA_ShowTitle, FALSE,
										SA_Quiet, TRUE,
										SA_AutoScroll, TRUE,
										SA_Font,(ULONG)ami_fnt,
										TAG_END, NULL);

	if (!ami_scr)
	{
		puts("Can`t open Amiga screen!");
		exit(5);
	}

	vp = &ami_scr->ViewPort;

	ami_win = OpenWindowTags(  NULL,
										WA_Left,0,
										WA_Top,0,
										WA_Width,font_w * WIN_COLS,
										WA_Height,font_h * WIN_ROWS,
										WA_IDCMP, IDCMP_RAWKEY,
										WA_Backdrop,TRUE,
										WA_Borderless,TRUE,
										WA_Activate,TRUE,
										WA_Flags,WFLG_RMBTRAP,
										WA_DepthGadget,TRUE,
										WA_CustomScreen,ami_scr,
										TAG_END );

	if (!ami_win)
	{
		CloseScreen(ami_scr);
		puts("Can`t open Amiga window");
		exit(5);
	}

	f = fopen("PROGDIR:Crawl.cfg", "r");
	if (!f)
	{
		fclose(f);

		f = fopen("PROGDIR:Crawl.cfg", "w");

		if (f)
		{
			fprintf(f,"%lx\n",reqmode);
			fprintf(f,"%s\n",amifont.ta_Name);
			fprintf(f,"%d\n",amifont.ta_YSize);

			fclose(f);
		}
	}

	rp = ami_win->RPort;
	SetFont(rp, ami_fnt);

	for (i = 0 ; i < 16 ; i++)
		setrgb(i,0xFF,0xFF,0xFF);

	/* These colours are mostly guesses! */
	setrgb(BLACK,		0x00,0x00,0x00);
	setrgb(BLUE,		0x00,0x00,0xE0);
	setrgb(GREEN,		0x00,0xE0,0x00);
	setrgb(CYAN,		0x04,0x40,0xE0);
	setrgb(RED,			0xE0,0x00,0x00);
	setrgb(MAGENTA, 	0xFF,0x00,0x60);
	setrgb(BROWN,		0xD0,0x60,0x00);
	setrgb(LIGHTGREY, 0xD0,0xD0,0xD0);
	setrgb(DARKGREY,	0x80,0x80,0x80);
	setrgb(LIGHTBLUE, 0x00,0xD0,0xFF);
	setrgb(LIGHTGREEN,0x00,0xE0,0xFF);
	setrgb(LIGHTCYAN, 0x20,0xA0,0xFF);
	setrgb(LIGHTRED,	0xFF,0x40,0x40);
	setrgb(LIGHTMAGENTA,0xFF,0x40,0x60);
	setrgb(YELLOW,		0xE0,0xE0,0x00);
	setrgb(WHITE,		0xFF,0xFF,0xFF);
	clrscr();
	gotoxy(0,0);
}

void lincurses_shutdown()
{
	ULONG iclass;
	UWORD icode;
	UWORD iqual;
	APTR iaddr;

	struct IntuiMessage *imsg;

	if (!start_state)
		return;
	if (ami_fnt && font_owner)
		CloseFont(ami_fnt);
	if (ami_win)
	{
		while (imsg = (struct IntuiMessage *)GetMsg( ami_win->UserPort ))
			ReplyMsg((struct Message *) imsg );
	}
	if (ami_win)
		CloseWindow(ami_win);
	if (ami_scr)
		CloseScreen(ami_scr);
	if ( ConsoleDev )
	{
		CloseDevice( (struct IORequest *)ConsoleDev );
		ConsoleDev = NULL;
	}

	if (AslBase)
		CloseLibrary( (struct Library *)AslBase);
	CloseLibrary((struct Library *)DiskfontBase);
	CloseLibrary((struct Library *)IntuitionBase);
	CloseLibrary((struct Library *)GfxBase);
	start_state = 0;
}

/* Convert value to string */
int itoa(int value, char *strptr, int radix)
{
	unsigned int bitmask = 32768;
	int ctr = 0;
	int startflag = 0;

	if (radix == 10)
		sprintf(strptr, "%i", value);
	else if (radix == 2)
	{
		while (bitmask)
		{
			if (value & bitmask)
			{
				startflag = 1;
				sprintf(strptr + ctr, "1");
			}
			else
			{
				if (startflag)
					sprintf(strptr + ctr, "0");
			}
			bitmask >>= 1;
			if (startflag)
				ctr++;
		}
		if (!startflag) /* Special case if value == 0 */
			sprintf((strptr + ctr++), "0");
		strptr[ctr] = (char) NULL;
	}
}

int cprintf (char *format, ... )
{
	int i;
	char buffer[2048]; // One full screen if no control seq...

	va_list argp;
	va_start(argp,format);
	vsprintf(buffer,format,argp);
	va_end(argp);

	print_text(buffer);
	return(strlen(buffer));
}

/* Ought to line wrap??? */
void print_text(char *s)
{
	while (*s)
		putch(*s++);
}

int putch(unsigned char chr)
{
	static char s[2];

	if (!chr)
		chr = 32;

	s[1] = 0;
	s[0] = chr;

	switch(chr)
	{
		case 10:
			gotoxy(0,my_y + 1);
//			gotoxy(my_x,my_y + 1);
			break;
		case 13:
			gotoxy(0,my_y);
			break;
		default:
			Text(rp,s,1);
			my_x++;
			if (my_y >= 30)
				my_y = 30;
			if (my_x == 80)
				gotoxy(0,my_y + 1);
			break;
	}
}

char getche()
{
	char c;

	putch( c = getch() );
	return c;
}

int window(int x1, int y1, int x2, int y2)
{
	return(0);
}

int clrscr()
{
	SetRast(rp,BLACK);
	SetAPen(rp,WHITE);
	gotoxy(0,0);
}

int stricmp(char *str1, char *str2)
{
	return(strcmp(str1, str2));
}

void _setcursortype(int curstype)
{
}

int textcolor(int col)
{
	short fg;

	fg = (FG_COL = col)& 0xF;

	SetAPen(rp,fg);
}

void textbackground(int col)
{
	short bg;

	bg = (BG_COL = col) & 0xF;
	SetBPen(rp,bg);
}

int gotoxy(int x, int y)
{
	if (y >= 30)
		y = 30;

	Move(rp,x * font_w, y * font_h + font_b);
	my_x = x;
	my_y = y;
}

int wherex()
{
	return( my_x );
}

int wherey()
{
	return( my_y );
}

/* Delay time ms, I think */
void delay(long time)
{
#ifdef PPC
	WaitTime(0, time * 1000);
#else
	usleep(time * 1000);
#endif
}


int kbhit()
{
	char s[20];
	ULONG iclass;
	UWORD icode;
	UWORD iqual;
	APTR iaddr;
	int len;

	struct IntuiMessage *imsg;

	imsg = (struct IntuiMessage *)GetMsg( ami_win->UserPort );

	if (imsg)
	{
		iclass = imsg->Class;
		iqual = imsg->Qualifier;
		iaddr = imsg->IAddress;
		icode = imsg->Code;

		ReplyMsg((struct Message *) imsg );

		if (iclass == IDCMP_RAWKEY)
		{
			ie.ie_Class = IECLASS_RAWKEY;
			ie.ie_Code = icode;
			ie.ie_Qualifier = iqual;
			ie.ie_EventAddress = (APTR *) *((ULONG *) iaddr );
			len = RawKeyConvert( &ie, s, 10, NULL );
			if (!len)
				return 0;
			pending = keycode;
			return 1;
		}
	}
	return 0;
}

char getch(void)
{
	char s[20];
	ULONG iclass;
	UWORD icode;
	UWORD iqual;
	APTR iaddr;
	int len;

	struct IntuiMessage *imsg;

	if (pending)
	{
		keycode = pending;
		pending = 0;
		return keycode;
	}

	do
	{
		Wait( 1 << ami_win->UserPort->mp_SigBit );
		imsg = (struct IntuiMessage *)GetMsg( ami_win->UserPort );
	   if (imsg)
		{
			iclass = imsg->Class;
			iqual = imsg->Qualifier;
			iaddr = imsg->IAddress;
			icode = imsg->Code;

			ReplyMsg((struct Message *) imsg );

			if (iclass == IDCMP_RAWKEY)
			{
				ie.ie_Class = IECLASS_RAWKEY;
				ie.ie_Code = icode;
				ie.ie_Qualifier = iqual;
				ie.ie_EventAddress = (APTR *) *((ULONG *) iaddr );
				len = RawKeyConvert( &ie, s, 10, NULL );
				if (!len)
					continue;
				keycode = s[0];
//				printf("%d\n",keycode);
				return keycode;
			}
      }
	} while (1); // Ouch.
}

char *strupr(char *s)
{
	for (char *p = s ; *p ; p++)
		*p = toupper(*p);

	return s;
}

int echo()
{
}

int noecho()
{
}

int getstr(char *s)
{
	char c;
	int i = 0;

  	do
	{
		c = getch();
		if (i && (c == 8 || c == 127))
		{
			gotoxy(my_x - 1,my_y);
			putch(32);
			gotoxy(my_x - 1,my_y);
			i--;
		}
      else if (c > 31)
		{
			s[i++] = c;
			putch(c);
		}
	} while (c > 31);
	s[i] = 0;
}

int random(void)
{
	return rand();
}

void srandom(unsigned int a)
{
	srand(a);
}

